﻿using Microsoft.Extensions.Caching.Memory;
using Newtonsoft.Json;
using System.Text;

namespace Sample
{
    public class BaiduApiService
    {
        private readonly IMemoryCache _cache;
        private readonly HttpClient _httpClient;
        private const string TokenUrl = "https://aip.baidubce.com/oauth/2.0/token";
        private const string ChatApiUrl = "https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat";
        private const string ClientId = "***********"; // 替换为实际的AK
        private const string ClientSecret = "***********"; // 替换为实际的SK
        private const string SteamStart = "data: ";


        public BaiduApiService(IMemoryCache memoryCache, IHttpClientFactory httpClientFactory)
        {
            _cache = memoryCache;
            _httpClient = httpClientFactory.CreateClient();
            _httpClient.Timeout = TimeSpan.FromMinutes(5);
        }

        public async Task<string> GetAccessTokenAsync()
        {
            if (_cache.TryGetValue("AccessToken", out string accessToken))
            {
                return accessToken;
            }

            var response = await _httpClient.PostAsync($"{TokenUrl}?grant_type=client_credentials&client_id={ClientId}&client_secret={ClientSecret}", null);
            response.EnsureSuccessStatusCode();

            var tokenResponse = await response.Content.ReadFromJsonAsync<TokenResponse>();
            if (tokenResponse != null && !string.IsNullOrEmpty(tokenResponse.access_token))
            {
                // 设置缓存，有效期30天
                _cache.Set("AccessToken", tokenResponse.access_token, TimeSpan.FromDays(30));
                return tokenResponse.access_token;
            }

            throw new Exception("Failed to retrieve access token.");
        }

        public async Task<BaiduAIResponse> CallChatApiAsync(BaiduChatRequest chatRequest, string model)
        {
            var accessToken = await GetAccessTokenAsync();
            var response = await _httpClient.PostAsJsonAsync($"{ChatApiUrl}/{model}?access_token={accessToken}", chatRequest);
            response.EnsureSuccessStatusCode();

            return await response.Content.ReadFromJsonAsync<BaiduAIResponse>();
        }

        public async IAsyncEnumerable<string> CallChatApiStreamAsync(BaiduChatRequest chatRequest, string model)
        {
            var accessToken = await GetAccessTokenAsync();
            using var httpRequest = new HttpRequestMessage();
            httpRequest.Method = HttpMethod.Post;
            httpRequest.RequestUri = new Uri($"{ChatApiUrl}/{model}?access_token={accessToken}");
            httpRequest.Content = new StringContent(JsonConvert.SerializeObject(chatRequest));
            using var cancellationTokenSource = new CancellationTokenSource(120 * 1000);

            using var httpResponseMessage = await _httpClient.SendAsync(httpRequest, HttpCompletionOption.ResponseHeadersRead, cancellationTokenSource.Token);
            using var stream = await httpResponseMessage.Content.ReadAsStreamAsync();
            using StreamReader reader = new(stream, Encoding.UTF8);
            string line;
            while ((line = await reader.ReadLineAsync()) != null)
            {
                if (line == string.Empty)
                    yield return "\r\n\r\n";
                else
                {
                    var oldLine = line.Replace(SteamStart, string.Empty);
                    var response = JsonConvert.DeserializeObject<BaiduAIResponse>(oldLine);
                    var newLineObj = new CompletionResponse { choices = [new Choice { delta = new RespMessage { content = response.result } }] };
                    var newLine = $"{SteamStart}{JsonConvert.SerializeObject(newLineObj)}";
                    if (response.is_end)
                    {
                        newLine += "\r\n\r\n" + $"{SteamStart}[DONE]";
                    }
                    yield return newLine;
                }
            }
        }

    }
}
